import { __extends, __assign, __rest, __read } from 'tslib';
import { ElementEvent, Shape, Path, CustomElement, PropertySyntax, CSS, deg2rad } from '@antv/g-lite';
import { isNil, isNumberEqual } from '@antv/util';
import { vec3 } from 'gl-matrix';

/**
 * support 3 types of arrow line:
 * 1. Line
 * 2. Polyline
 * 3. Path
 *
 * support 2 types of arrow head:
 * 1. default(Path)
 * 2. custom
 */
var Arrow = /** @class */ (function (_super) {
    __extends(Arrow, _super);
    function Arrow(config) {
        var _this = _super.call(this, __assign(__assign({}, config), { type: Arrow.tag })) || this;
        var _a = _this.attributes, body = _a.body, startHead = _a.startHead, endHead = _a.endHead; _a.startHeadOffset; _a.endHeadOffset; var rest = __rest(_a, ["body", "startHead", "endHead", "startHeadOffset", "endHeadOffset"]);
        if (!body) {
            throw new Error("Arrow's body is required");
        }
        // append arrow body
        _this.body = body;
        _this.appendChild(_this.body);
        _this.handleBodyAttributeChanged(_this.body);
        if (startHead) {
            _this.appendArrowHead(_this.getArrowHeadType(startHead), true);
        }
        if (endHead) {
            _this.appendArrowHead(_this.getArrowHeadType(endHead), false);
        }
        _this.applyArrowStyle(rest, [_this.body, _this.startHead, _this.endHead]);
        return _this;
    }
    Arrow.prototype.handleBodyAttributeChanged = function (body) {
        var _this = this;
        body.addEventListener(ElementEvent.ATTR_MODIFIED, function (e) {
            var attrName = e.attrName;
            if (attrName === 'x1' || attrName === 'y1') {
                if (_this.startHead) {
                    _this.transformArrowHead(_this.startHead, true);
                }
            }
            else if (attrName === 'x2' || attrName === 'y2') {
                if (_this.endHead) {
                    _this.transformArrowHead(_this.endHead, false);
                }
            }
            // const { nodeName } = body;
            // if (nodeName === Shape.LINE) {
            //   body
            // }
        });
    };
    Arrow.prototype.getBody = function () {
        return this.body;
    };
    Arrow.prototype.getStartHead = function () {
        return this.startHead;
    };
    Arrow.prototype.getEndHead = function () {
        return this.endHead;
    };
    Arrow.prototype.attributeChangedCallback = function (name, oldValue, newValue) {
        var _a;
        if (name === 'opacity' ||
            name === 'strokeOpacity' ||
            name === 'stroke' ||
            name === 'lineWidth' ||
            name === 'increasedLineWidthForHitTesting') {
            this.applyArrowStyle((_a = {}, _a[name] = newValue, _a), [
                this.body,
                this.startHead,
                this.endHead,
            ]);
        }
        else if (name === 'startHead' || name === 'endHead') {
            var isStart = name === 'startHead';
            // delete existed arrow head first
            this.destroyArrowHead(isStart);
            if (newValue) {
                var _b = this.attributes; _b.body; _b.startHead; _b.endHead; _b.startHeadOffset; _b.endHeadOffset; var rest = __rest(_b, ["body", "startHead", "endHead", "startHeadOffset", "endHeadOffset"]);
                // append new arrow head
                this.appendArrowHead(this.getArrowHeadType(newValue), isStart);
                this.applyArrowStyle(rest, [isStart ? this.startHead : this.endHead]);
            }
        }
        else if (name === 'body') {
            var _c = this.attributes; _c.body; _c.startHead; _c.endHead; _c.startHeadOffset; _c.endHeadOffset; var rest = __rest(_c, ["body", "startHead", "endHead", "startHeadOffset", "endHeadOffset"]);
            this.body.destroy();
            // @ts-ignore
            this.body = newValue;
            this.appendChild(this.body);
            this.applyArrowStyle(rest, [this.body]);
        }
        else if (name === 'startHeadOffset') {
            this.moveArrowHeadAlongTangent(newValue, true);
        }
        else if (name === 'endHeadOffset') {
            this.moveArrowHeadAlongTangent(newValue, false);
        }
    };
    Arrow.prototype.getArrowHeadType = function (head) {
        if (typeof head === 'boolean') {
            return 'default';
        }
        return 'custom';
    };
    Arrow.prototype.appendArrowHead = function (type, isStart) {
        var head;
        if (type === 'default') {
            head = this.createDefaultArrowHead();
        }
        else {
            // @ts-ignore
            head = isStart ? this.attributes.startHead : this.attributes.endHead;
        }
        // set position & rotation
        this.transformArrowHead(head, isStart);
        // heads should display on top of body
        head.setAttribute('zIndex', 1);
        if (isStart) {
            this.startHead = head;
        }
        else {
            this.endHead = head;
        }
        this.appendChild(head);
        var offset = isStart
            ? this.attributes.startHeadOffset
            : this.attributes.endHeadOffset;
        if (offset) {
            this.moveArrowHeadAlongTangent(offset, isStart);
        }
    };
    /**
     * transform arrow head according to arrow line
     */
    Arrow.prototype.transformArrowHead = function (head, isStart) {
        var _a, _b;
        var position = vec3.create();
        var rad = 0;
        var x1 = 0;
        var x2 = 0;
        var y1 = 0;
        var y2 = 0;
        var bodyType = this.body && this.body.nodeName;
        if (bodyType === Shape.LINE) {
            var _x1 = (_a = this.body.attributes, _a.x1), _x2 = _a.x2, _y1 = _a.y1, _y2 = _a.y2;
            x1 = isStart ? _x2 : _x1;
            x2 = isStart ? _x1 : _x2;
            y1 = isStart ? _y2 : _y1;
            y2 = isStart ? _y1 : _y2;
        }
        else if (bodyType === Shape.POLYLINE) {
            var points = this.body.attributes.points;
            var length_1 = points.length;
            x1 = isStart ? points[1][0] : points[length_1 - 2][0];
            y1 = isStart ? points[1][1] : points[length_1 - 2][1];
            x2 = isStart ? points[0][0] : points[length_1 - 1][0];
            y2 = isStart ? points[0][1] : points[length_1 - 1][1];
        }
        else if (bodyType === Shape.PATH) {
            var p1 = (_b = __read(this.getTangent(this.body, isStart), 2), _b[0]), p2 = _b[1];
            x1 = p1[0];
            y1 = p1[1];
            x2 = p2[0];
            y2 = p2[1];
        }
        var x = x1 - x2;
        var y = y1 - y2;
        rad = Math.atan2(y, x);
        position = vec3.fromValues(x2, y2, 0);
        if (isStart) {
            this.startHeadPosition = position;
            this.startHeadRad = rad;
        }
        else {
            this.endHeadPosition = position;
            this.endHeadRad = rad;
        }
        head.setLocalPosition(position);
        head.setLocalEulerAngles((rad * 180) / Math.PI + head.getLocalEulerAngles());
    };
    Arrow.prototype.moveArrowHeadAlongTangent = function (offset, isStart) {
        var head = isStart ? this.startHead : this.endHead;
        if (head) {
            head.setLocalPosition(vec3.sub(vec3.create(), isStart ? this.startHeadPosition : this.endHeadPosition, vec3.fromValues(Math.cos(isStart ? this.startHeadRad : this.endHeadRad) * offset, Math.sin(isStart ? this.startHeadRad : this.endHeadRad) * offset, 0)));
        }
        // cut body
        if (this.body) ;
    };
    Arrow.prototype.destroyArrowHead = function (isStart) {
        if (isStart && this.startHead) {
            this.startHead.destroy();
            this.startHead = undefined;
        }
        if (!isStart && this.endHead) {
            this.endHead.destroy();
            this.endHead = undefined;
        }
    };
    Arrow.prototype.getTangent = function (path, isStart) {
        return isStart ? path.getStartTangent() : path.getEndTangent();
    };
    Arrow.prototype.createDefaultArrowHead = function () {
        var _a;
        var stroke = (_a = this.attributes, _a.stroke), lineWidth = _a.lineWidth;
        var sin = Math.sin, cos = Math.cos, PI = Math.PI;
        return new Path({
            style: {
                // draw an angle '<'
                path: "M".concat(10 * cos(PI / 6), ",").concat(10 * sin(PI / 6), " L0,0 L").concat(10 * cos(PI / 6), ",-").concat(10 * sin(PI / 6)),
                stroke: stroke,
                lineWidth: lineWidth,
                transformOrigin: 'center',
                anchor: [0.5, 0.5], // set anchor to center
            },
        });
    };
    Arrow.prototype.applyArrowStyle = function (attributes, objects) {
        var opacity = attributes.opacity, stroke = attributes.stroke, strokeOpacity = attributes.strokeOpacity, lineWidth = attributes.lineWidth, increasedLineWidthForHitTesting = attributes.increasedLineWidthForHitTesting;
        objects.forEach(function (shape) {
            if (shape) {
                if (!isNil(opacity)) {
                    shape.style.opacity = opacity;
                }
                if (!isNil(stroke)) {
                    shape.style.stroke = stroke;
                }
                if (!isNil(strokeOpacity)) {
                    shape.style.strokeOpacity = strokeOpacity;
                }
                if (!isNil(lineWidth)) {
                    shape.style.lineWidth = lineWidth;
                }
                if (!isNil(increasedLineWidthForHitTesting)) {
                    shape.style.increasedLineWidthForHitTesting =
                        increasedLineWidthForHitTesting;
                }
            }
        });
    };
    Arrow.tag = 'arrow';
    return Arrow;
}(CustomElement));

var PI = Math.PI;
var PI2 = PI * 2;
var mathSin = Math.sin;
var mathCos = Math.cos;
var mathACos = Math.acos;
var mathATan2 = Math.atan2;
// const mathAbs = Math.abs;
var mathSqrt = Math.sqrt;
var mathMax = Math.max;
var mathMin = Math.min;
var e = 1e-4;
// 注册 css 属性
var SECTOR_CSS_PROPERTY = [
    {
        name: 'sr',
        inherits: false,
        interpolable: true,
        syntax: PropertySyntax.LENGTH_PERCENTAGE,
    },
    {
        name: 'sr0',
        inherits: false,
        interpolable: true,
        syntax: PropertySyntax.LENGTH_PERCENTAGE,
    },
    {
        name: 'startAngle',
        inherits: false,
        interpolable: true,
        syntax: PropertySyntax.ANGLE,
    },
    {
        name: 'endAngle',
        inherits: false,
        interpolable: true,
        syntax: PropertySyntax.ANGLE,
    },
];
SECTOR_CSS_PROPERTY.forEach(function (property) {
    CSS.registerProperty(property);
});
var polarToCartesian = function (centerX, centerY, radius, angleInRadian) {
    return {
        x: centerX + radius * Math.cos(angleInRadian),
        y: centerY + radius * Math.sin(angleInRadian),
    };
};
function intersect(x0, y0, x1, y1, x2, y2, x3, y3) {
    var dx10 = x1 - x0;
    var dy10 = y1 - y0;
    var dx32 = x3 - x2;
    var dy32 = y3 - y2;
    var t = dy32 * dx10 - dx32 * dy10;
    if (t * t < e) {
        return;
    }
    t = (dx32 * (y0 - y2) - dy32 * (x0 - x2)) / t;
    return [x0 + t * dx10, y0 + t * dy10];
}
// Compute perpendicular offset line of length rc.
function computeCornerTangents(x0, y0, x1, y1, radius, cr, clockwise) {
    var x01 = x0 - x1;
    var y01 = y0 - y1;
    var lo = (clockwise ? cr : -cr) / mathSqrt(x01 * x01 + y01 * y01);
    var ox = lo * y01;
    var oy = -lo * x01;
    var x11 = x0 + ox;
    var y11 = y0 + oy;
    var x10 = x1 + ox;
    var y10 = y1 + oy;
    var x00 = (x11 + x10) / 2;
    var y00 = (y11 + y10) / 2;
    var dx = x10 - x11;
    var dy = y10 - y11;
    var d2 = dx * dx + dy * dy;
    var r = radius - cr;
    var s = x11 * y10 - x10 * y11;
    var d = (dy < 0 ? -1 : 1) * mathSqrt(mathMax(0, r * r * d2 - s * s));
    var cx0 = (s * dy - dx * d) / d2;
    var cy0 = (-s * dx - dy * d) / d2;
    var cx1 = (s * dy + dx * d) / d2;
    var cy1 = (-s * dx + dy * d) / d2;
    var dx0 = cx0 - x00;
    var dy0 = cy0 - y00;
    var dx1 = cx1 - x00;
    var dy1 = cy1 - y00;
    // Pick the closer of the two intersection points
    // TODO: Is there a faster way to determine which intersection to use?
    if (dx0 * dx0 + dy0 * dy0 > dx1 * dx1 + dy1 * dy1) {
        cx0 = cx1;
        cy0 = cy1;
    }
    return {
        cx: cx0,
        cy: cy0,
        x0: -ox,
        y0: -oy,
        x1: cx0 * (radius / r - 1),
        y1: cy0 * (radius / r - 1),
    };
}
function computeArcSweep(startAngle, endAngle) {
    endAngle = endAngle < 0 && startAngle >= 0 ? endAngle + PI2 : endAngle;
    return endAngle - startAngle <= PI ? 0 : 1;
}
var Sector = /** @class */ (function (_super) {
    __extends(Sector, _super);
    // parsedStyle: any;
    function Sector(config) {
        var _this = _super.call(this, config) || this;
        _this.updatePath();
        return _this;
    }
    Sector.prototype.setAttribute = function (name, value, force) {
        _super.prototype.setAttribute.call(this, name, value, force);
        if (['startAngle', 'endAngle', 'sr', 'sr0', 'radius'].indexOf(name) > -1) {
            this.updatePath();
        }
    };
    Sector.prototype.updatePath = function () {
        // @ts-ignore
        var _a = this.parsedStyle, x = _a.x, y = _a.y, startAngle = _a.startAngle, endAngle = _a.endAngle, sr = _a.sr, sr0 = _a.sr0, radius = _a.radius;
        var path = this.createPath(x, y, startAngle ? deg2rad(startAngle) : 0, endAngle ? deg2rad(endAngle) : Math.PI * 2, sr ? sr : 0, sr0 ? sr0 : 0, radius ? radius : [0, 0, 0, 0]);
        _super.prototype.setAttribute.call(this, 'path', path);
    };
    Sector.prototype.createPath = function (x, y, startAngle, endAngle, r, r0, borderRadius) {
        if (r <= 0) {
            return;
        }
        var start = polarToCartesian(x, y, r, startAngle);
        var end = polarToCartesian(x, y, r, endAngle);
        var innerStart = polarToCartesian(x, y, r0, startAngle);
        var innerEnd = polarToCartesian(x, y, r0, endAngle);
        // 整圆
        if (isNumberEqual(endAngle - startAngle, Math.PI * 2)) {
            // 整个圆是分割成两个圆
            var middlePoint = polarToCartesian(x, y, r, startAngle + Math.PI);
            var innerMiddlePoint = polarToCartesian(x, y, r0, startAngle + Math.PI);
            var circlePathCommands = [
                ['M', start.x, start.y],
                ['A', r, r, 0, 1, 1, middlePoint.x, middlePoint.y],
                ['A', r, r, 0, 1, 1, end.x, end.y],
                ['M', innerStart.x, innerStart.y],
            ];
            if (r0) {
                circlePathCommands.push([
                    'A',
                    r0,
                    r0,
                    0,
                    1,
                    0,
                    innerMiddlePoint.x,
                    innerMiddlePoint.y,
                ]);
                circlePathCommands.push(['A', r0, r0, 0, 1, 0, innerEnd.x, innerEnd.y]);
            }
            circlePathCommands.push(['M', start.x, start.y]);
            circlePathCommands.push(['Z']);
            return circlePathCommands;
        }
        var angle = endAngle - startAngle;
        var xrs = r * mathCos(startAngle);
        var yrs = r * mathSin(startAngle);
        var xire = r0 * mathCos(endAngle);
        var yire = r0 * mathSin(endAngle);
        var xre = r * mathCos(endAngle);
        var yre = r * mathSin(endAngle);
        var xirs = r0 * mathCos(startAngle);
        var yirs = r0 * mathSin(startAngle);
        // 顺时针反向，同 radius
        var _a = __read(borderRadius, 4), outStartRadius = _a[0], outEndRadius = _a[1], innerEndRadius = _a[2], innerStartRadius = _a[3];
        var halfRadius = (r - r0) / 2;
        var outStartBorderRadius = mathMin(halfRadius, outStartRadius);
        var outEndBorderRadius = mathMin(halfRadius, outEndRadius);
        var innerEndBorderRadius = mathMin(halfRadius, innerEndRadius);
        var innerStartBorderRadius = mathMin(halfRadius, innerStartRadius);
        var outBorderRadiusMax = mathMax(outStartBorderRadius, outEndBorderRadius);
        var innerBorderRadiusMax = mathMax(innerEndBorderRadius, innerStartBorderRadius);
        var limitedOutBorderRadiusMax = outBorderRadiusMax;
        var limitedInnerBorderRadiusMax = innerBorderRadiusMax;
        // draw corner radius
        if (outBorderRadiusMax > e || innerBorderRadiusMax > e) {
            // restrict the max value of corner radius
            if (angle < PI) {
                var it_1 = intersect(xrs, yrs, xirs, yirs, xre, yre, xire, yire);
                if (it_1) {
                    var x0 = xrs - it_1[0];
                    var y0 = yrs - it_1[1];
                    var x1 = xre - it_1[0];
                    var y1 = yre - it_1[1];
                    var a = 1 /
                        mathSin(mathACos((x0 * x1 + y0 * y1) /
                            (mathSqrt(x0 * x0 + y0 * y0) * mathSqrt(x1 * x1 + y1 * y1))) / 2);
                    var b = mathSqrt(it_1[0] * it_1[0] + it_1[1] * it_1[1]);
                    limitedOutBorderRadiusMax = mathMin(outBorderRadiusMax, (r - b) / (a + 1));
                    limitedInnerBorderRadiusMax = mathMin(innerBorderRadiusMax, (r0 - b) / (a - 1));
                }
            }
        }
        var arcSweep = computeArcSweep(startAngle, endAngle);
        var clockwise = true;
        var sectorPathCommands = [];
        if (limitedOutBorderRadiusMax > e) {
            var crStart = mathMin(outStartRadius, limitedOutBorderRadiusMax);
            var crEnd = mathMin(outEndRadius, limitedOutBorderRadiusMax);
            var ct0 = computeCornerTangents(xirs, yirs, xrs, yrs, r, crStart, clockwise);
            var ct1 = computeCornerTangents(xre, yre, xire, yire, r, crEnd, clockwise);
            sectorPathCommands.push(['M', x + ct0.cx + ct0.x0, y + ct0.cy + ct0.y0]);
            // Have the corners merged?
            if (limitedOutBorderRadiusMax < outBorderRadiusMax && crStart === crEnd) {
                var outStartBorderRadiusStartAngle = mathATan2(ct0.y0, ct0.x0);
                var outStartBorderRadiusEndAngle = mathATan2(ct1.y0, ct1.x0);
                sectorPathCommands.push([
                    'A',
                    limitedOutBorderRadiusMax,
                    limitedOutBorderRadiusMax,
                    0,
                    computeArcSweep(outStartBorderRadiusStartAngle, outStartBorderRadiusEndAngle),
                    1,
                    x + ct1.cx + ct1.x0,
                    y + ct1.cy + ct1.y0,
                ]);
            }
            else {
                // draw the two corners and the ring
                if (crStart > 0) {
                    var outStartBorderRadiusStartAngle = mathATan2(ct0.y0, ct0.x0);
                    var outStartBorderRadiusEndAngle = mathATan2(ct0.y1, ct0.x1);
                    var outStartBorderRadiusEndPoint = polarToCartesian(x, y, r, outStartBorderRadiusEndAngle);
                    sectorPathCommands.push([
                        'A',
                        crStart,
                        crStart,
                        0,
                        computeArcSweep(outStartBorderRadiusStartAngle, outStartBorderRadiusEndAngle),
                        1,
                        outStartBorderRadiusEndPoint.x,
                        outStartBorderRadiusEndPoint.y,
                    ]);
                }
                var outRadiusStartAngle = mathATan2(ct0.cy + ct0.y1, ct0.cx + ct0.x1);
                var outRadiusEndAngle = mathATan2(ct1.cy + ct1.y1, ct1.cx + ct1.x1);
                var outRadiusEndPoint = polarToCartesian(x, y, r, outRadiusEndAngle);
                sectorPathCommands.push([
                    'A',
                    r,
                    r,
                    0,
                    computeArcSweep(outRadiusStartAngle, outRadiusEndAngle),
                    1,
                    outRadiusEndPoint.x,
                    outRadiusEndPoint.y,
                ]);
                if (crEnd > 0) {
                    var outEndBorderRadiusStartAngle = mathATan2(ct1.y1, ct1.x1);
                    var outEndBorderRadiusEndAngle = mathATan2(ct1.y0, ct1.x0);
                    sectorPathCommands.push([
                        'A',
                        crEnd,
                        crEnd,
                        0,
                        computeArcSweep(outEndBorderRadiusStartAngle, outEndBorderRadiusEndAngle),
                        1,
                        x + ct1.cx + ct1.x0,
                        y + ct1.cy + ct1.y0,
                    ]);
                }
            }
        }
        else {
            sectorPathCommands.push(['M', start.x, start.y]);
            sectorPathCommands.push(['A', r, r, 0, arcSweep, 1, end.x, end.y]);
        }
        // no inner ring, is a circular sector
        if (r0 < e) {
            sectorPathCommands.push(['L', innerEnd.x, innerEnd.y]);
        }
        // the inner ring has corners
        else if (limitedInnerBorderRadiusMax > e) {
            var crStart = mathMin(innerStartRadius, limitedInnerBorderRadiusMax);
            var crEnd = mathMin(innerEndRadius, limitedInnerBorderRadiusMax);
            var ct0 = computeCornerTangents(xire, yire, 0, 0, r0 - r, crEnd, clockwise);
            var ct1 = computeCornerTangents(0, 0, xirs, yirs, r0 - r, crStart, clockwise);
            sectorPathCommands.push(['L', x + ct0.cx + ct0.x0, y + ct0.cy + ct0.y0]);
            // Have the corners merged?
            if (limitedInnerBorderRadiusMax < innerBorderRadiusMax &&
                crStart === crEnd) {
                var innerStartBorderRadiusStartAngle = mathATan2(ct0.y0, ct0.x0);
                var innerStartBorderRadiusEndAngle = mathATan2(ct1.y0, ct1.x0);
                var innerStartBorderRadiusEndPoint = polarToCartesian(x, y, r0, innerStartBorderRadiusEndAngle);
                sectorPathCommands.push([
                    'A',
                    limitedOutBorderRadiusMax,
                    limitedOutBorderRadiusMax,
                    0,
                    computeArcSweep(innerStartBorderRadiusStartAngle, innerStartBorderRadiusEndAngle),
                    1,
                    innerStartBorderRadiusEndPoint.x,
                    innerStartBorderRadiusEndPoint.y,
                ]);
            }
            // draw the two corners and the ring
            else {
                if (crEnd > 0) {
                    var innerStartBorderRadiusStartAngle = mathATan2(ct0.y0, ct0.x0);
                    var innerStartBorderRadiusEndAngle = mathATan2(ct0.y1, ct0.x1);
                    var innerStartBorderRadiusEndPoint = polarToCartesian(x, y, r0 - r, innerStartBorderRadiusEndAngle);
                    sectorPathCommands.push([
                        'A',
                        crEnd,
                        crEnd,
                        0,
                        computeArcSweep(innerStartBorderRadiusStartAngle, innerStartBorderRadiusEndAngle),
                        1,
                        innerStartBorderRadiusEndPoint.x,
                        innerStartBorderRadiusEndPoint.y,
                    ]);
                }
                var innerRadiusStartAngle = mathATan2(ct0.cy + ct0.y1, ct0.cx + ct0.x1);
                var innerRadiusEndAngle = mathATan2(ct1.cy + ct1.y1, ct1.cx + ct1.x1);
                var innerRadiusEndPoint = polarToCartesian(x, y, r0, innerRadiusEndAngle);
                sectorPathCommands.push([
                    'A',
                    r0,
                    r0,
                    0,
                    computeArcSweep(innerRadiusEndAngle, innerRadiusStartAngle),
                    0,
                    innerRadiusEndPoint.x,
                    innerRadiusEndPoint.y,
                ]);
                sectorPathCommands.push([
                    'L',
                    innerRadiusEndPoint.x,
                    innerRadiusEndPoint.y,
                ]);
                if (crStart > 0) {
                    var innerEndBorderRadiusStartAngle = mathATan2(ct1.y1, ct1.x1);
                    var innerEndBorderRadiusEndAngle = mathATan2(ct1.y0, ct1.x0);
                    sectorPathCommands.push([
                        'A',
                        crStart,
                        crStart,
                        0,
                        computeArcSweep(innerEndBorderRadiusStartAngle, innerEndBorderRadiusEndAngle),
                        1,
                        x + ct1.cx + ct1.x0,
                        y + ct1.cy + ct1.y0,
                    ]);
                }
            }
        }
        // the inner ring is just a circular arc
        else {
            sectorPathCommands.push(['L', innerEnd.x, innerEnd.y]);
            sectorPathCommands.push([
                'A',
                r0,
                r0,
                0,
                arcSweep,
                0,
                innerStart.x,
                innerStart.y,
            ]);
        }
        sectorPathCommands.push(['Z']);
        return sectorPathCommands;
    };
    return Sector;
}(Path));

export { Arrow, Sector };
//# sourceMappingURL=index.esm.js.map
